@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -99,8 +99,8 @@ static uint64_t cli_rawaddr64(uint64_t vaddr, struct elf_program_hdr64 *ph, uint
 }
 
 /* Return converted endian-fixed header, or error code */
-static int cli_elf_fileheader(cli_ctx *ctx, fmap_t *map, union elf_file_hdr *file_hdr,
-                              uint8_t *do_convert, uint8_t *is64)
+static cl_error_t cli_elf_fileheader(cli_ctx *ctx, fmap_t *map, union elf_file_hdr *file_hdr,
+                                     uint8_t *do_convert, uint8_t *is64)
 {
     uint8_t format64, conv;
 
@@ -127,9 +127,10 @@ static int cli_elf_fileheader(cli_ctx *ctx, fmap_t *map, union elf_file_hdr *fil
             break;
         default:
             cli_dbgmsg("ELF: Unknown ELF class (%u)\n", file_hdr->hdr64.e_ident[4]);
-            if (ctx)
-                cli_append_virus(ctx, "Heuristics.Broken.Executable");
-            return CL_VIRUS;
+            if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
+                return CL_VIRUS;
+            }
+            return CL_BREAK;
     }
 
     /* Need to know to endian convert */
@@ -219,8 +220,7 @@ static int cli_elf_ph32(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
     cli_dbgmsg("ELF: Number of program headers: %d\n", phnum);
     if (phnum > 128) {
         cli_dbgmsg("ELF: Suspicious number of program headers\n");
-        if (ctx && SCAN_HEURISTIC_BROKEN) {
-            cli_append_virus(ctx, "Heuristics.Broken.Executable");
+        if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
             return CL_VIRUS;
         }
         return CL_EFORMAT;
@@ -232,8 +232,7 @@ static int cli_elf_ph32(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
         /* Sanity check */
         if (phentsize != sizeof(struct elf_program_hdr32)) {
             cli_dbgmsg("ELF: phentsize != sizeof(struct elf_program_hdr32)\n");
-            if (ctx && SCAN_HEURISTIC_BROKEN) {
-                cli_append_virus(ctx, "Heuristics.Broken.Executable");
+            if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
                 return CL_VIRUS;
             }
             return CL_EFORMAT;
@@ -267,8 +266,7 @@ static int cli_elf_ph32(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
                     cli_dbgmsg("ELF: Possibly broken ELF file\n");
                 }
                 free(program_hdr);
-                if (ctx && SCAN_HEURISTIC_BROKEN) {
-                    cli_append_virus(ctx, "Heuristics.Broken.Executable");
+                if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
                     return CL_VIRUS;
                 }
                 return CL_BREAK;
@@ -289,8 +287,7 @@ static int cli_elf_ph32(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
         free(program_hdr);
         if (err) {
             cli_dbgmsg("ELF: Can't calculate file offset of entry point\n");
-            if (ctx && SCAN_HEURISTIC_BROKEN) {
-                cli_append_virus(ctx, "Heuristics.Broken.Executable");
+            if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
                 return CL_VIRUS;
             }
             return CL_EFORMAT;
@@ -309,8 +306,8 @@ static int cli_elf_ph32(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
 }
 
 /* Read 64-bit program headers */
-static int cli_elf_ph64(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
-                        struct elf_file_hdr64 *file_hdr, uint8_t conv)
+static cl_error_t cli_elf_ph64(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
+                               struct elf_file_hdr64 *file_hdr, uint8_t conv)
 {
     struct elf_program_hdr64 *program_hdr = NULL;
     uint16_t phnum, phentsize;
@@ -323,8 +320,7 @@ static int cli_elf_ph64(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
     cli_dbgmsg("ELF: Number of program headers: %d\n", phnum);
     if (phnum > 128) {
         cli_dbgmsg("ELF: Suspicious number of program headers\n");
-        if (ctx && SCAN_HEURISTIC_BROKEN) {
-            cli_append_virus(ctx, "Heuristics.Broken.Executable");
+        if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
             return CL_VIRUS;
         }
         return CL_EFORMAT;
@@ -336,8 +332,7 @@ static int cli_elf_ph64(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
         /* Sanity check */
         if (phentsize != sizeof(struct elf_program_hdr64)) {
             cli_dbgmsg("ELF: phentsize != sizeof(struct elf_program_hdr64)\n");
-            if (ctx && SCAN_HEURISTIC_BROKEN) {
-                cli_append_virus(ctx, "Heuristics.Broken.Executable");
+            if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
                 return CL_VIRUS;
             }
             return CL_EFORMAT;
@@ -371,8 +366,7 @@ static int cli_elf_ph64(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
                     cli_dbgmsg("ELF: Possibly broken ELF file\n");
                 }
                 free(program_hdr);
-                if (ctx && SCAN_HEURISTIC_BROKEN) {
-                    cli_append_virus(ctx, "Heuristics.Broken.Executable");
+                if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
                     return CL_VIRUS;
                 }
                 return CL_BREAK;
@@ -393,8 +387,7 @@ static int cli_elf_ph64(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
         free(program_hdr);
         if (err) {
             cli_dbgmsg("ELF: Can't calculate file offset of entry point\n");
-            if (ctx && SCAN_HEURISTIC_BROKEN) {
-                cli_append_virus(ctx, "Heuristics.Broken.Executable");
+            if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
                 return CL_VIRUS;
             }
             return CL_EFORMAT;
@@ -437,8 +430,7 @@ static int cli_elf_sh32(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
     /* Sanity check */
     if (shentsize != sizeof(struct elf_section_hdr32)) {
         cli_dbgmsg("ELF: shentsize != sizeof(struct elf_section_hdr32)\n");
-        if (ctx && SCAN_HEURISTIC_BROKEN) {
-            cli_append_virus(ctx, "Heuristics.Broken.Executable");
+        if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
             return CL_VIRUS;
         }
         return CL_EFORMAT;
@@ -481,8 +473,7 @@ static int cli_elf_sh32(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
                 cli_dbgmsg("ELF: Possibly broken ELF file\n");
             }
             free(section_hdr);
-            if (ctx && SCAN_HEURISTIC_BROKEN) {
-                cli_append_virus(ctx, "Heuristics.Broken.Executable");
+            if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
                 return CL_VIRUS;
             }
             return CL_BREAK;
@@ -538,8 +529,7 @@ static int cli_elf_sh64(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
     /* Sanity check */
     if (shentsize != sizeof(struct elf_section_hdr64)) {
         cli_dbgmsg("ELF: shentsize != sizeof(struct elf_section_hdr64)\n");
-        if (ctx && SCAN_HEURISTIC_BROKEN) {
-            cli_append_virus(ctx, "Heuristics.Broken.Executable");
+        if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
             return CL_VIRUS;
         }
         return CL_EFORMAT;
@@ -582,8 +572,7 @@ static int cli_elf_sh64(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
                 cli_dbgmsg("ELF: Possibly broken ELF file\n");
             }
             free(section_hdr);
-            if (ctx && SCAN_HEURISTIC_BROKEN) {
-                cli_append_virus(ctx, "Heuristics.Broken.Executable");
+            if (ctx && SCAN_HEURISTIC_BROKEN && (CL_VIRUS == cli_append_potentially_unwanted(ctx, "Heuristics.Broken.Executable"))) {
                 return CL_VIRUS;
             }
             return CL_BREAK;
@@ -683,11 +672,11 @@ static void cli_elf_sectionlog(uint32_t sh_type, uint32_t sh_flags)
 }
 
 /* Scan function for ELF */
-int cli_scanelf(cli_ctx *ctx)
+cl_error_t cli_scanelf(cli_ctx *ctx)
 {
     union elf_file_hdr file_hdr;
-    fmap_t *map = *ctx->fmap;
-    int ret;
+    fmap_t *map = ctx->fmap;
+    cl_error_t ret;
     uint8_t conv = 0, is64 = 0;
 
     cli_dbgmsg("in cli_scanelf\n");
@@ -802,11 +791,11 @@ int cli_scanelf(cli_ctx *ctx)
 /* ELF header parsing only
  * Returns 0 on success, -1 on error
  */
-int cli_elfheader(fmap_t *map, struct cli_exe_info *elfinfo)
+cl_error_t cli_elfheader(cli_ctx *ctx, struct cli_exe_info *elfinfo)
 {
     union elf_file_hdr file_hdr;
     uint8_t conv = 0, is64 = 0;
-    int ret;
+    cl_error_t ret = CL_SUCCESS;
 
     cli_dbgmsg("in cli_elfheader\n");
 
@@ -816,86 +805,87 @@ int cli_elfheader(fmap_t *map, struct cli_exe_info *elfinfo)
         cli_dbgmsg("cli_elfheader: Assumption Violated: elfinfo->offset != 0\n");
     }
 
-    ret = cli_elf_fileheader(NULL, map, &file_hdr, &conv, &is64);
-    if (ret != CL_CLEAN) {
-        return -1;
+    ret = cli_elf_fileheader(NULL, ctx->fmap, &file_hdr, &conv, &is64);
+    if (ret != CL_SUCCESS) {
+        goto done;
     }
 
     /* Program headers and Entry */
     if (is64) {
-        ret = cli_elf_ph64(NULL, map, elfinfo, &(file_hdr.hdr64), conv);
+        ret = cli_elf_ph64(NULL, ctx->fmap, elfinfo, &(file_hdr.hdr64), conv);
     } else {
-        ret = cli_elf_ph32(NULL, map, elfinfo, &(file_hdr.hdr32.hdr), conv);
+        ret = cli_elf_ph32(NULL, ctx->fmap, elfinfo, &(file_hdr.hdr32.hdr), conv);
     }
-    if (ret != CL_CLEAN) {
-        return -1;
+    if (ret != CL_SUCCESS) {
+        goto done;
     }
 
     /* Section Headers */
     if (is64) {
-        ret = cli_elf_sh64(NULL, map, elfinfo, &(file_hdr.hdr64), conv);
+        ret = cli_elf_sh64(NULL, ctx->fmap, elfinfo, &(file_hdr.hdr64), conv);
     } else {
-        ret = cli_elf_sh32(NULL, map, elfinfo, &(file_hdr.hdr32.hdr), conv);
+        ret = cli_elf_sh32(NULL, ctx->fmap, elfinfo, &(file_hdr.hdr32.hdr), conv);
     }
-    if (ret != CL_CLEAN) {
-        return -1;
+    if (ret != CL_SUCCESS) {
+        goto done;
     }
 
-    return 0;
+done:
+
+    return ret;
 }
 
 /*
  * ELF file unpacking.
  */
-int cli_unpackelf(cli_ctx *ctx)
+cl_error_t cli_unpackelf(cli_ctx *ctx)
 {
-    char *tempfile;
-    int ndesc;
+    cl_error_t ret = CL_SUCCESS;
+    char *tempfile = NULL;
+    int ndesc      = -1;
     struct cli_bc_ctx *bc_ctx;
-    int ret;
-    fmap_t *map = *ctx->fmap;
 
     /* Bytecode BC_ELF_UNPACKER hook */
     bc_ctx = cli_bytecode_context_alloc();
     if (!bc_ctx) {
         cli_errmsg("cli_scanelf: can't allocate memory for bc_ctx\n");
-        return CL_EMEM;
+        ret = CL_EMEM;
+        goto done;
     }
 
     cli_bytecode_context_setctx(bc_ctx, ctx);
 
     cli_dbgmsg("Running bytecode hook\n");
-    ret = cli_bytecode_runhook(ctx, ctx->engine, bc_ctx, BC_ELF_UNPACKER, map);
+    ret = cli_bytecode_runhook(ctx, ctx->engine, bc_ctx, BC_ELF_UNPACKER, ctx->fmap);
     cli_dbgmsg("Finished running bytecode hook\n");
-    switch (ret) {
-        case CL_VIRUS:
-            cli_bytecode_context_destroy(bc_ctx);
-            return CL_VIRUS;
-        case CL_SUCCESS:
-            ndesc = cli_bytecode_context_getresult_file(bc_ctx, &tempfile);
-            cli_bytecode_context_destroy(bc_ctx);
-            if (ndesc != -1 && tempfile) {
-                if (ctx->engine->keeptmp)
-                    cli_dbgmsg("cli_scanelf: Unpacked and rebuilt executable saved in %s\n", tempfile);
-                else
-                    cli_dbgmsg("cli_scanelf: Unpacked and rebuilt executable\n");
-                lseek(ndesc, 0, SEEK_SET);
-                cli_dbgmsg("***** Scanning rebuilt ELF file *****\n");
-                if (cli_magic_scandesc(ndesc, tempfile, ctx) == CL_VIRUS) {
-                    close(ndesc);
-                    CLI_TMPUNLK();
-                    free(tempfile);
-                    return CL_VIRUS;
-                }
-                close(ndesc);
-                CLI_TMPUNLK();
-                free(tempfile);
-                return CL_CLEAN;
-            }
-            break;
-        default:
-            cli_bytecode_context_destroy(bc_ctx);
+    if (CL_SUCCESS == ret) {
+        // check for unpacked/rebuilt executable
+        ndesc = cli_bytecode_context_getresult_file(bc_ctx, &tempfile);
+        if (ndesc != -1 && tempfile) {
+            cli_dbgmsg("cli_scanelf: Unpacked and rebuilt ELF executable saved in %s\n", tempfile);
+
+            lseek(ndesc, 0, SEEK_SET);
+
+            cli_dbgmsg("***** Scanning rebuilt ELF file *****\n");
+            ret = cli_magic_scan_desc(ndesc, tempfile, ctx, NULL, LAYER_ATTRIBUTES_NONE);
+        }
     }
 
-    return CL_CLEAN;
+done:
+    // cli_bytecode_context_getresult_file() gives up ownership of temp file, so we must clean it up.
+    if (-1 != ndesc) {
+        close(ndesc);
+    }
+    if (NULL != tempfile) {
+        if (!ctx->engine->keeptmp) {
+            (void)cli_unlink(tempfile);
+        }
+        free(tempfile);
+    }
+
+    if (NULL != bc_ctx) {
+        cli_bytecode_context_destroy(bc_ctx);
+    }
+
+    return ret;
 }
