@@ -1,7 +1,7 @@
 /*
  *  Unit tests for JS normalizer.
  *
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
@@ -28,15 +28,17 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "../libclamav/clamav.h"
-#include "../libclamav/uniq.h"
+// libclamav
+#include "clamav.h"
+#include "uniq.h"
+
 #include "checks.h"
 
 START_TEST(test_uniq_initfail)
 {
     struct uniq *U;
     U = uniq_init(0);
-    fail_unless(U == NULL, "uniq_init(0)!=NULL");
+    ck_assert_msg(U == NULL, "uniq_init(0)!=NULL");
 }
 END_TEST
 
@@ -58,20 +60,20 @@ START_TEST(test_uniq_known)
     int i;
 
     struct uniq *U = uniq_init(5);
-    fail_unless(U != 0, "uniq_init");
+    ck_assert_msg(U != 0, "uniq_init");
 
     for (i = 0; tests[i].expected; i++) {
         if (CL_SUCCESS != uniq_add(U, tests[i].key, tests[i].key_len, &hash, &u)) {
-            fail("uniq_add(%s) failed.", tests[i].key);
+            ck_abort_msg("uniq_add(%s) failed.", tests[i].key);
         }
-        fail_unless_fmt(u == 1 && strcmp(hash, tests[i].expected) == 0, "uniq_add(%s) = %u - expected %s, got %s", tests[i].key, u, tests[i].expected, hash);
+        ck_assert_msg(u == 1 && strcmp(hash, tests[i].expected) == 0, "uniq_add(%s) = %u - expected %s, got %s", tests[i].key, u, tests[i].expected, hash);
     }
 
     for (i = 0; tests[i].expected; i++) {
         if (CL_SUCCESS != uniq_get(U, tests[i].key, tests[i].key_len, &hash, &u)) {
-            fail("uniq_get(%s) failed.", tests[i].key);
+            ck_abort_msg("uniq_get(%s) failed.", tests[i].key);
         }
-        fail_unless_fmt(u == 1 && strcmp(hash, tests[i].expected) == 0, "uniq_get(%s) = %u - expected %s, got %s", tests[i].key, u, tests[i].expected, hash);
+        ck_assert_msg(u == 1 && strcmp(hash, tests[i].expected) == 0, "uniq_get(%s) = %u - expected %s, got %s", tests[i].key, u, tests[i].expected, hash);
     }
 
     uniq_free(U);
@@ -85,20 +87,20 @@ START_TEST(test_uniq_colls)
     int i, j;
 
     struct uniq *U = uniq_init(10);
-    fail_unless(U != 0, "uniq_init");
+    ck_assert_msg(U != 0, "uniq_init");
 
     for (j = 4; j > 0; j--)
         for (i = 0; i < j; i++) {
             if (CL_SUCCESS != uniq_add(U, tests[i], strlen(tests[i]), NULL, &u)) {
-                fail("uniq_add(%s) failed.", tests[i]);
+                ck_abort_msg("uniq_add(%s) failed.", tests[i]);
             }
         }
 
     for (i = 0; i < 4; i++) {
         if (CL_SUCCESS != uniq_get(U, tests[i], strlen(tests[i]), NULL, &u)) {
-            fail("uniq_get(%s) failed.", tests[i]);
+            ck_abort_msg("uniq_get(%s) failed.", tests[i]);
         }
-        fail_unless_fmt(u + i == 4, "uniq_get(%s) = %u - expected %u", tests[i], u, 4 - i);
+        ck_assert_msg(u + i == 4, "uniq_get(%s) = %u - expected %u", tests[i], u, 4 - i);
     }
 
     uniq_free(U);
